<?php
$config = array();

// Path to DiggerDiff files
// Must end with '/'
$config['homepath'] = './';

// Path to saved pages
// Must end with '/'
$config['pagepath'] = "pages/";

// Path to current version of pages, relative to 'pagepath'/'urlfile'/.
// Must end with '/'
$config['oldpath'] = "old/";

// Path to latest version of pages, relative to 'pagepath'/'urlfile'/.
// Must end with '/'
$config['newpath'] = "new/";

// Path to directory for temp files
$config['tmpdir'] = '/usr/tmp';

// URL to load in blank frame of split main window
$config['blankurl'] = 'docs.html';

// Command to use for diff
$config['diffcmd'] = "/usr/bin/diff -bBsU 0";

// Command to use for zdiff (for compressed files)
// diff options must be inside of '' for zdiff
$config['zdiffcmd'] = "/usr/bin/zdiff '-bBsU 0'";

// Command to use for lynx
$config['lynxcmd'] = '/usr/local/bin/lynx -source';

// Command to use for curl
$config['curlcmd'] = '/usr/local/bin/curl --silent --location';

// Command to use for HTTPS URLs
$config['httpscmd'] = $config['curlcmd'];

// Command to use for POST URLs
//   curl --form = multipart/form-data
//   curl --data = application/x-www-form-urlencoded
$config['postcmd'] = $config['curlcmd'] . " --data";

// Compress local copies of pages
$config['compress'] = FALSE;

// If less than $config['agelimit'] seconds have passed since fetching a page,
// don't bother fetching it again.  Just use local copy.
$config['agelimit'] = 3600;

// Set how sites are checked for updates:  demand or cron
$config['updatemode'] = 'demand';

// Actions to take when run by cron.  Options are 'fetch', 'email' and 'save'.
// If web is primary interface, either just 'fetch' or 'fetch' and 'email' are
// sensible settings.  If email alerts are the primary interface, 'fetch' and
// 'email' and 'save' are probably all desired.
$config['cronactions'] = array('fetch', 'email');

// URL files to process when run by cron
$config['cronfiles'] = array('urls');

// Email contents to send.  Sent in the order specified, except 'diffs'.
// If 'diffs' is specified, the diff of all CHANGED pages are included as
// separate attachments.  Options are 'fetch', 'new', 'same', 'error', 
// 'changed', 'save', 'diffs'.
$config['emailcontents'] = array('fetch', 'new', 'same', 'changed', 'error');

// Address to send email to
$config['emailto'] = '';

// Address to send email from
$config['emailfrom'] = '';


//========================================


// Session defaults:
$config['default'] = array();

// URL file, relative to 'pagepath', '.dat' extension added.
// Alphanumeric only.
$config['default']['urlfile'] = "urls";

// Page number to display
$config['default']['page'] = 1;

// String to filter site titles with
$config['default']['filtersite'] = '';

// Format mode for diff display
// 'br' = use <br>, 'pre' = use <pre>
$config['default']['diffsourcemode'] = 'pre';
$config['default']['diffrendermode'] = 'br';

// Main window layout:  'single', 'split'
$config['default']['mainlayout'] = 'split';

// Split main window divider:  'horiz', 'vert'
$config['default']['maindivider'] = 'vert';

// Main window frame size
// Percentage for the first (left or top) frame.
$config['default']['mainsize'] = 45;

// Diff frame layout:  'single', 'split'
$config['default']['difflayout'] = 'split';

// Split diff frame divider:  'horiz', 'vert'
$config['default']['diffdivider'] = 'horiz';

// Single diff style:  'render', 'source'
$config['default']['diffstyle'] = 'render';

// Diff frame size
// Setting for the first (left or top) frame.  Other frame is 100 minus this.
$config['default']['diffsize'] = 75;

// Show diff lines beginning with these chars
// For regular 'diff', '123456789<>-' should show all lines
// For 'diff --context' or 'diff -c', '*! +-' should show all lines
// For 'diff --context=0' or 'diff -C 0', '*!+-' should show all lines
// For 'diff --unified' or 'diff -u', '@ +-' should show all lines
// For 'diff --unified=0' or 'diff -U 0', '@+-' should show all lines
$config['default']['diffshow'] = '@+-';

// Number of items to list on each page.
// Best to keep this relatively low when $config['updatemode']='demand',
// since it takes time to fetch each one.
$config['default']['perpage'] = 20;

// Order of row display
// Choices are 'blank', 'config', 'edit', 'filter', 'list', 'pages', 'save', 'search'
$config['default']['rows'] = array('pages', 'list', 'blank', 'pages', 'filter', 'search', 'config', 'save', 'edit');

// Order of column display in 'list' section
// Choices are 'archive', 'blank', 'current', 'edit', 'fetch', 'save', 'site', 'status'
$config['default']['columns'] = array('edit', 'site', 'fetch', 'current', 'save', 'archive', 'status');

// Sort the URLs in the list?
$config['default']['sorturls'] = TRUE;

// Display comments in list?
$config['default']['comments'] = FALSE;

// Display page ranges?
$config['default']['ranges'] = TRUE;

// Width of select list for editing
$config['default']['editselectwidth'] = 80;

// Lines in select list for editing
$config['default']['editselectlines'] = 18;

// Width of edit fields
$config['default']['editwidth'] = 80;

// Lines in edit fields
$config['default']['editlines'] = 10;

?>
