<?php
// LICENSE
/*
DiggerDiff v1.2
Copyright (c) 2003, Dave Walton
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// END LICENSE


function action_edit($config, $mode, $linenum, $linedata, $site) {
  global $HTTP_SESSION_VARS;
  $urlfile = $HTTP_SESSION_VARS['urlfile'];

  $filename = $config['pagepath'] . "$urlfile.dat";
  $contents = load_file($filename, FALSE, FALSE);
  foreach ($contents as $key => $line) {
    $contents[$key] = trim($line);
  }

  echo print_header('strict', '- Edit List');
  echo "<body>\n";

  switch ($mode) {
    case 'Copy Line':
      $contents = action_edit_insert('copy', $filename, $linenum, $contents);
      if ($contents !== FALSE) {
        action_edit_edit($linenum+1, $contents, $site);
      }
      break;
    case 'Delete Line':
      action_edit_delete($filename, $linenum, $contents);
      break;
    case 'Edit Line':
      action_edit_edit($linenum, $contents, $site);
      break;
    case 'Insert Line':
      $contents = action_edit_insert('insert', $filename, $linenum, $contents);
      if ($contents !== FALSE) {
        action_edit_edit($linenum, $contents, $site);
      }
      break;
    case 'Move Up':
      $contents = action_edit_move('up', $filename, $linenum, $contents);
      if ($contents !== FALSE) {
        action_edit_default($config, $linenum-1, $contents);
      }
      break;
    case 'Move Down':
      $contents = action_edit_move('down', $filename, $linenum, $contents);
      if ($contents !== FALSE) {
        action_edit_default($config, $linenum+1, $contents);
      }
      break;
    case 'Save Changes':
      action_edit_save($config, $filename, $linenum, $linedata, $contents, $site);
      break;
    default:
      action_edit_default($config, $linenum, $contents);
      break;
  }

  echo "</body>\n";
  echo "</html>\n";
}


function action_edit_default($config, $linenum, $contents) {
  global $HTTP_SERVER_VARS, $HTTP_SESSION_VARS;
  $sid = strip_tags(SID);
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];
  $editselectlines = $HTTP_SESSION_VARS['editselectlines'];
  $editselectwidth = $HTTP_SESSION_VARS['editselectwidth'];

  echo "<form method=\"post\" action=\"$php_self\">\n";
  echo "  <div>\n";
  echo "  <input type=\"hidden\" name=\"" . session_name() . "\" value=\"" . session_id() . "\" />\n";
  echo "  <input type=\"hidden\" name=\"action\" value=\"edit\" />\n";
  echo "  <select name=\"line\" size=\"$editselectlines\">\n";
  foreach ($contents as $key => $line) {
    $key++;
    echo "    <option value=\"$key\"";
    if ($linenum === $key) {
      echo " selected=\"selected\"";
    }
    echo ">";
    if (strlen($line) === 0) {
      $output = '';
    } else if ($line[0] === '#') {
      if (strlen($line) > $editselectwidth) {
        $output = substr($line, 0, 77) . '...';
      } else {
        $output = $line;
      }
    } else {
      // only display up to 4 fields (req. PHP 4.0.1)
      //$fields = explode(chr(0x09), $line, 4);
      // in PHP < 4.0.1, loop array_pop()
      $fields = explode(chr(0x09), $line);
      $count = count($fields);
      while (count($fields) > 4) {
        array_pop($fields);
      }
      if ($count > count($fields)) {
        $fields[] = '...';
        $count = count($fields);
      }
      $maxlen = $editselectwidth - (($count-1) * 4);
      $totallen = 0;
      foreach ($fields as $field) {
        $totallen += strlen($field);
      }
      while ($totallen > $maxlen) {
        $longest = 0;
        foreach ($fields as $field) {
          $longest = (strlen($field) > $longest) ? strlen($field) : $longest;
        }
        foreach ($fields as $fieldnum => $field) {
          if (strlen($field) === $longest) {
            $fields[$fieldnum] = substr($field, 0, -4) . '...';
          }
        }
        $totallen = 0;
        foreach ($fields as $field) {
          $totallen += strlen($field);
        }
      }
      $output = '';
      foreach ($fields as $field) {
        if (!empty($output)) {
          $output .= ' || ';
        }
        $output .= $field;
      }
    }
    $output = htmlspecialchars($output);
    echo "$output</option>\n";
  }
  echo "  </select>\n";
  echo "  <br />\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Insert Line\" />\n";
  echo "  &nbsp; &nbsp;\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Copy Line\" />\n";
  echo "  &nbsp; &nbsp;\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Delete Line\" />\n";
  echo "  &nbsp; &nbsp;\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Move Up\" />\n";
  echo "  &nbsp; &nbsp;\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Move Down\" />\n";
  echo "  &nbsp; &nbsp;\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Edit Line\" />\n";
  echo "  &nbsp; &nbsp;\n";
  echo "  <a href=\"javascript:window.close();\">Close window</a>\n";
  echo "  </div>\n";
  echo "</form>\n";
  return(TRUE);
}


function action_edit_delete($filename, $linenum, $contents) {
  global $HTTP_SERVER_VARS;
  $sid = strip_tags(SID);
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];

  echo "<p>\n";
  if (($linenum < 1) || ($linenum > count($contents))) {
    echo "Invalid line number $linenum.<br />\n";
    echo "<a href=\"$php_self?$sid&amp;action=edit\">Continue editing...</a><br />\n";
    return(FALSE);
  }
  $output = $contents[$linenum-1];
  unset($contents[$linenum-1]);
  $contents = implode("\n", $contents);
  save_file($filename, $contents, FALSE);
  echo "Line $linenum deleted:<br />\n";
  echo "$output<br />\n";
  echo "<a href=\"$php_self?$sid&amp;action=edit&amp;line=" . ($linenum-1) . "\">Continue editing...</a><br />\n";
  echo "</p>\n";
  return(TRUE);
}


function action_edit_edit($linenum, $contents, $site) {
  global $HTTP_SERVER_VARS, $HTTP_SESSION_VARS;
  $sid = strip_tags(SID);
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];
  $editlines = $HTTP_SESSION_VARS['editlines'];
  $editwidth = $HTTP_SESSION_VARS['editwidth'];

  if (($linenum === 0) && (count($contents) === 0)) {
    // handle insert on empty file
    $contents[0] = '';
    $linenum = 1;
  }
  if ((($linenum < 1) && empty($site)) || ($linenum > count($contents))) {
    echo "<p>\n";
    echo "Invalid line number $linenum.<br />\n";
    echo "<a href=\"$php_self?$sid&amp;action=edit\">Continue editing...</a><br />\n";
    echo "</p>\n";
    return(FALSE);
  } else if ($linenum < 1) {
    // Ugh.  gotta do some work to find the right line.
    foreach ($contents as $key => $line) {
      $linedata = parse_line($line);
      if ($linedata['md5'] === $site) {
        $linenum = $key + 1;
        break;
      }
    }
    if ($linenum < 1) {
      echo "<p>\n";
      echo "Invalid site '$site'.<br />\n";
      echo "<a href=\"$php_self?$sid&amp;action=list\">Return to list...</a><br />\n";
      echo "</p>\n";
      return(FALSE);
    }
  }
  $linedata = $contents[$linenum-1];
  if (empty($linedata) || ($linedata[0] === '#')) {
    $title = '';
    $url = $linedata;
    $postdata = '';
    $regexes = array();
  } else {
    $linedata = parse_line($linedata);
    $title = $linedata['title'];
    $url = $linedata['url'];
    $postdata = $linedata['postdata'];
    $regexes = $linedata['regex'];
  }

  echo "<form method=\"post\" action=\"$php_self\">\n";
  echo "  <div>\n";
  echo "  <input type=\"hidden\" name=\"" . session_name() . "\" value=\"" . session_id() . "\" />\n";
  echo "  <input type=\"hidden\" name=\"action\" value=\"edit\" />\n";
  echo "  <input type=\"hidden\" name=\"line\" value=\"$linenum\" />\n";
  if (!empty($site)) {
    echo "  <input type=\"hidden\" name=\"site\" value=\"$site\" />\n";
  }
  echo "  Title:<br />\n";
  echo "  <input type=\"text\" size=\"$editwidth\" name=\"linedata[title]\" value=\"";
  echo htmlspecialchars($title) . "\" />\n";
  echo "  <br />\n";
  echo "  URL / Comment:<br />\n";
  echo "  <input type=\"text\" size=\"$editwidth\" name=\"linedata[url]\" value=\"";
  echo htmlspecialchars($url) . "\" />\n";
  echo "  <br />\n";
  echo "  Post Data:<br />\n";
  echo "  <input type=\"text\" size=\"$editwidth\" name=\"linedata[postdata]\" value=\"";
  echo htmlspecialchars($postdata) . "\" />\n";
  echo "  <br />\n";
  echo "  Search and Replace:<br />\n";
  echo "  <textarea rows=\"$editlines\" cols=\"$editwidth\" name=\"linedata[regex]\">";
  foreach ($regexes as $regex) {
    echo htmlspecialchars($regex) . "\n";
  }
  echo "</textarea>\n";
  echo "  <br />\n";
  echo "  <input type=\"reset\" value=\"Undo Changes\" />\n";
  echo "  <input type=\"submit\" name=\"mode\" value=\"Save Changes\" />\n";
  echo "  </div>\n";
  echo "</form>\n\n";
  echo "<p>\n";
  echo "<br />\n";
  echo "<b>Field descriptions:</b>\n";
  echo "</p>\n";
  echo "<dl>\n";
  echo "  <dt>Title</dt>\n";
  echo "  <dd>This is the name that will be displayed for the site in the site\n";
  echo "      list.  This field can only be empty if 'Post Data' and 'Search and\n";
  echo "      Replace' are also empty.  In other words, when 'URL/Comment' is\n";
  echo "      the only non-empty field, or ALL fields are empty.  To convert a\n";
  echo "      regular entry into a comment, put a '#' as the first character of\n";
  echo "      this field.</dd>\n";
  echo "  <dt>URL / Comment</dt>\n";
  echo "  <dd>This is the URL for the page to be monitored, or a comment if this\n";
  echo "      entry is a comment.  To make a comment, fill out only this field,\n";
  echo "      using '#' as the first character.  This is the only mandatory\n";
  echo "      field, and MUST have an entry unless ALL fields are blank.</dd>\n";
  echo "  <dt>Post Data</dt>\n";
  echo "  <dd>This field contains data to be POSTed to URL, if URL requires\n";
  echo "      a POST.  The data will be sent as-is, so be sure that it is\n";
  echo "      properly encoded for a POST.</dd>\n";
  echo "  <dt>Search and Replace</dt>\n";
  echo "  <dd>These are expressions that will be matched against the contents\n";
  echo "      of the page.  They can be used to strip out predictable, unimportant\n";
  echo "      changes from a page, or to reformat a page to produce a clearer\n";
  echo "      diff.  The expression format is: \"/search/[replace]/[modifiers]\".\n";
  echo "      You can enter multiple expressions here, one expression per line,\n";
  echo "      and one line per expression. The separator can be any character,\n";
  echo "      not just '/', but MUST appear EXACTLY three times in the expression.<br />\n";
  echo "      'Search' is a <a href=\"http://www.php.net/manual/en/pcre.pattern.syntax.php\" target=\"_blank\">pattern</a>\n";
  echo "      that is matched against the contents of a page.  Matching is done\n";
  echo "      with PHP's <a href=\"http://www.php.net/manual/en/function.preg-replace.php\" target=\"_blank\">preg_replace()</a>\n";
  echo "      function, so this pattern must fit the rules for that function.<br />\n";
  echo "      If 'replace' is blank, anything matching 'search' will\n";
  echo "      be removed, otherwise matching text will be replaced with the\n";
  echo "      contents of 'replace'.  As with 'search', this must fit the rules\n";
  echo "      for preg_replace().  In addition, certain special characters must\n";
  echo "      be properly escaped to be processed by <a href=\"http://www.php.net/manual/en/function.stripcslashes.php\" target=\"_blank\">stripcslashes()</a>.<br />\n";
  echo "      'Modifiers' is an optional parameter containing Pattern Modifiers\n";
  echo "      for use in preg_replace().  The possible modifiers are listed\n";
  echo "      <a href=\"http://www.php.net/manual/en/pcre.pattern.modifiers.php\" target=\"_blank\">here</a>.\n";
  echo "      Among others, 'i' makes the search case insensitive, 'm' MUST be\n";
  echo "      used if you use '^' (beginning of line) or '$' (end of line) in\n";
  echo "      your search, 's' makes '.' include newlines, and 'U' makes the pattern\n";
  echo "      non-greedy.<br />\n";
  echo "      To sum up, an expression '/search/replace/modifiers' is used like\n";
  echo "      this:<br />\n";
  echo "      \$page = preg_replace('/search/modifiers', stripcslashes('replace'), \$page);\n";
  echo "      </dd>\n";
  echo "</dl>\n";
  return(TRUE);
}


function action_edit_insert($mode, $filename, $linenum, $contents) {
  global $HTTP_SERVER_VARS;
  $sid = strip_tags(SID);
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];

  if (($linenum === 0) && (count($contents) === 0)) {
    // handle insert on empty file
    return($contents);
  } else {
    if (($linenum < 1) || ($linenum > count($contents))) {
      echo "<p>\n";
      echo "Invalid line number $linenum.<br />\n";
      echo "<a href=\"$php_self?$sid&amp;action=edit\">Continue editing...</a><br />\n";
      echo "</p>\n";
      return(FALSE);
    }
    for ($key=count($contents); $key>($linenum-1); $key--) {
      $contents[$key] = $contents[$key-1];
    }
    if ($mode === 'copy') {
      if (!empty($contents[$linenum]) && ($contents[$linenum][0] !== '#')) {
        $linedata = parse_line($contents[$linenum]);
        $search = empty($linedata['url']) ? $linedata['title'] : $linedata['url'];
        if (!empty($search)) {
          $replace = "Copy of $search";
          $contents[$linenum] = str_replace($search, $replace, $contents[$linenum]);
        }
      }
    } else {
      $contents[$linenum-1] = '';
    }
  }
  save_file($filename, implode("\n", $contents), FALSE);
  echo "<p>\n";
  if ($mode === 'copy') {
    echo "Line $linenum copied.<br />\n";
  } else {
    echo "New line inserted before line $linenum.<br />\n";
  }
  echo "</p>\n";
  return($contents);
}


function action_edit_move($mode, $filename, $linenum, $contents) {
  global $HTTP_SERVER_VARS;
  $sid = strip_tags(SID);
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];

  $lastline = count($contents);
  if (($linenum < 1) || ($linenum > $lastline)) {
    echo "<p>\n";
    echo "Invalid line number $linenum.<br />\n";
    echo "<a href=\"$php_self?$sid&amp;action=edit\">Continue editing...</a><br />\n";
    echo "</p>\n";
    return(FALSE);
  }
  echo "<p>\n";
  switch ($mode) {
    case 'up':
      if ($linenum > 1) {
        $templine = $contents[$linenum-1];
        $contents[$linenum-1] = $contents[$linenum-2];
	$contents[$linenum-2] = $templine;
	echo "Line $linenum moved up.<br />\n";
      } else {
        echo "Line $linenum is already at the top.<br />\n";
      }
      break;
    case 'down':
    default:
      if ($linenum < $lastline) {
        $templine = $contents[$linenum-1];
        $contents[$linenum-1] = $contents[$linenum];
	$contents[$linenum] = $templine;
	echo "Line $linenum moved down.<br />\n";
      } else {
        echo "Line $linenum is already at the bottom.<br />\n";
      }
      break;
  }
  save_file($filename, implode("\n", $contents), FALSE);
  echo "</p>\n";
  return($contents);
}


function action_edit_save($config, $filename, $linenum, $linedata, $contents, $site) {
  global $HTTP_SERVER_VARS;
  $sid = strip_tags(SID);
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];

  if (($linenum === 1) && (count($contents) === 0)) {
    // handle insert on empty file
    $contents[0] = '';
  }
  echo "<p>\n";
  if (($linenum < 1) || ($linenum > count($contents))) {
    echo "Invalid line number $linenum.<br />\n";
    if (!empty($site)) {
      echo "<a href=\"$php_self?$sid&amp;action=list\">Return to list...</a><br />\n";
    } else {
      echo "<a href=\"$php_self?$sid&amp;action=edit\">Continue editing...</a><br />\n";
    }
    return(FALSE);
  }
  $title = trim($linedata['title']);
  $url = trim($linedata['url']);
  $postdata = trim($linedata['postdata']);
  $regex = trim($linedata['regex']);
  // TODO:  is \r\n universal, or should I be doing line-ending checks???
  $regex = str_replace("\r\n", chr(0x09), $regex);
  if (empty($title) && empty($postdata) && empty($regex)) {
    if (empty($url)) {
      $linedata = '';
    } else {
      $linedata = $url;
    }
  } else {
    if (empty($url)) {
      if (!empty($title) && ($title[0] === '#')) {
      } else {
        echo "Error!  URL cannot be blank.  Please go back and try again.\n";
        echo "</p>\n";
	return(FALSE);
      }
    }
    if (empty($title)) {
      $linedata = $url;
    } else {
      $linedata = $title;
    }
    $linedata .= "\t$url";
    if (!empty($postdata)) {
      $linedata .= "\tPOST $postdata";
    }
    if (!empty($regex)) {
      $linedata .= "\t$regex";
    }
  }
  if (get_magic_quotes_gpc()) {
    $linedata = stripslashes($linedata);
  }
  $linedata = trim($linedata);
  $oldlinedata = parse_line($contents[$linenum-1]);
  $newlinedata = parse_line($linedata);
  if ($oldlinedata['md5'] !== $newlinedata['md5']) {
    echo action_edit_rename($config, $oldlinedata['md5'], $newlinedata['md5']);
  }
  $contents[$linenum-1] = $linedata;
  $contents = implode("\n", $contents);
  save_file($filename, $contents, FALSE);
  if (strlen($linedata) === 0) {
    echo "Line $linenum (blank line) saved:<br />\n";
    echo htmlspecialchars($linedata) . "<br />\n";
  } else if ($linedata[0] === '#') {
    echo "Line $linenum (comment) saved:<br />\n";
    echo htmlspecialchars($linedata) . "<br />\n";
  } else {
    $count = substr_count($linedata, chr(0x09)) + 1;
    echo "Line $linenum ($count fields) saved:<br />\n";
    echo htmlspecialchars($linedata) . "<br />\n";
    $urlmd5 = md5($url . $postdata);
    $sitedata = load_urls($config);
    foreach ($sitedata as $key => $data) {
      if ($data['title'][0] === '#') {
        continue;
      }
      if ($data['md5'] === $urlmd5) {
        $agelimit = $config['agelimit'];
        $config['agelimit'] = 0;
        echo fetch_pages($key, $key, $config, $sitedata);
        $config['agelimit'] = $agelimit;
        break;
      }
    }
  }
  if (!empty($site)) {
//    echo "<br />\n<a href=\"$php_self?$sid&amp;action=list\">Return to list...</a><br />\n";
    include_once('./dd_list.php');
    echo action_list($config);

  } else {
    echo "<br />\n<a href=\"$php_self?$sid&amp;action=edit&amp;line=$linenum\">Continue editing...</a><br />\n";
  }
  echo "</p>\n";
  return(TRUE);
}


function action_edit_rename($config, $oldsite, $newsite) {
  global $HTTP_SESSION_VARS;
  $urlfile = $HTTP_SESSION_VARS['urlfile'];

  $old_oldfile = build_filename('old', $config, $urlfile, $oldsite);
  $old_newfile = build_filename('new', $config, $urlfile, $oldsite);
  $new_oldfile = build_filename('old', $config, $urlfile, $newsite);
  $new_newfile = build_filename('new', $config, $urlfile, $newsite);

  $output = '';
  if (is_file($old_oldfile)) {
    if (rename($old_oldfile, $new_oldfile)) {
      $output .= "Archived local copy renamed.<br />\n";
    }
  }
  if (is_file($old_newfile)) {
    if (rename($old_newfile, $new_newfile)) {
      $output .= "Current local copy renamed.<br />\n";
    }
  }

  return($output);
}

?>
