<?php
// LICENSE
/*
DiggerDiff v1.2
Copyright (c) 2003, Dave Walton
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
// END LICENSE


function action_save($config, $action, $site='') {
  global $HTTP_SERVER_VARS, $HTTP_SESSION_VARS;
  $php_self = $HTTP_SERVER_VARS['PHP_SELF'];
  $urlfile = $HTTP_SESSION_VARS['urlfile'];
  $filtersite = $HTTP_SESSION_VARS['filtersite'];
  $page = $HTTP_SESSION_VARS['page'];
  $sid = strip_tags(SID);

  $return = '';
  $sitedata = load_urls($config);

  if ($action !== 'cron') {
    if (empty($site)) {
      $return .= print_header('strict', '- Save State');
    } else {
      // The ONLY reason this is Transitional instead of Strict is that
      // Strict doesn't allow the 'target' attribute of the 'a' tag.
      // (Of course, use of '&' instead of '&amp;' in links in the URL
      // list are an issue for both standards.)
      $return .= print_header('trans', "- View List - Page $page");
    }
    $return .= "<body>\n";
  }

  // make sure $config['newpath'] and $config['oldpath'] exist
  $dirname = $config['pagepath'];
  if (!is_dir($dirname)) {
    mkdir($dirname, 0755);
  }
  $dirname .= "$urlfile/";
  if (!is_dir($dirname)) {
    mkdir($dirname, 0755);
  }
  if (!is_dir($dirname . $config['newpath'])) {
    mkdir($dirname . $config['newpath'], 0755);
  }
  if (!is_dir($dirname . $config['oldpath'])) {
    mkdir($dirname . $config['oldpath'], 0755);
  }

  $goodfiles = array();

  $output = '';
  foreach ($sitedata as $data) {
    if ($data['title'][0] === '#') {
      continue;
    }
    $urlmd5 = $data['md5'];
    // Make sure to add all current sites to $goodfiles even if they are skipped, 
    // and go completely through $sitedata without breaking.  Otherwise the cleanup
    // below will delete valid files thinking they are obsolete.
    $goodfiles[] = $urlmd5;
    if (empty($site)) {
      // If no site was specified, skip any sites that don't match the filter.
      if (!empty($filtersite)) {
        if (stristr($data['title'], $filtersite) === FALSE) {
	  continue;
	}
      }
    } else {
      // If a site was specified, and this site doesn't match it, skip it.
      if ($urlmd5 !== $site) {
        continue;
      }
    }
    $oldfile = build_filename('old', $config, $urlfile, $urlmd5);
    $newfile = build_filename('new', $config, $urlfile, $urlmd5);
    $oldcontents = load_file($oldfile, $config['compress'], '');
    $newcontents = load_file($newfile, $config['compress'], '');
    if (($newcontents !== FALSE) && !empty($newcontents)) {
      $oldmd5 = md5($oldcontents);
      $newmd5 = md5($newcontents);
      if ($oldcontents === FALSE) {
        $newoutput = 'NEW: [' . $data['title'] . ']';
      } else if ($oldmd5 !== $newmd5) {
        $newoutput = 'CHANGED: [' . $data['title'] . ']';
      } else {
        $newoutput = '';
      }
      if (!empty($newoutput)) {
        $output .= (empty($site)) ? "$newoutput<br />\n" : "$newoutput\n";
        copy($newfile, $oldfile);
      }
    }
  }

  if (empty($site)) {
    $return .= "<dl>\n";
    $return .= "<dt>Saving NEW and CHANGED pages ";
    $return .= empty($filtersite) ? '' : "matching filter '$filtersite' ";
    $return .= "to archive...</dt>\n";
    $return .= "<dd>$output</dd>\n";
    $return .= "</dl>\n";
  } else {
    if (!empty($output)) {
      $return .= "<p>\n";
      $return .= "Saving $output";
      $return .= "</p>\n";
    }
  }

  $output = '';
  $dirlist = array($config['oldpath'], $config['newpath']);
  foreach ($dirlist as $dirname) {
    $dirname = $config['pagepath'] . "$urlfile/" . $dirname;
    $dir = opendir($dirname);
    while (FALSE !== ($filename = readdir($dir))) {
      if (preg_match('/^([^\W_]{32})$/', $filename, $matches) > 0) {
        if (!in_array($filename, $goodfiles)) {
	  $output .= "$dirname$filename<br />\n";
          unlink("$dirname$filename");
        }
      }
    }
    closedir($dir);
  }

  if (!empty($output)) {
    $return .= "<dl>\n";
    $return .= "  <dt>Deleting obsolete files...</dt>\n";
    $return .= "  <dd>$output</dd>\n";
    $return .= "</dl>\n";
  }

  if ($action !== 'cron') {
    if (empty($site)) {
      $return .= "<p>\n";
      $return .= "<a href=\"$php_self?$sid&amp;action=list\">Return to site list.</a>\n";
      $return .= "</p>\n";
      $return .= "</body>\n";
      $return .= "</html>\n";
    } else {
      include_once('./dd_list.php');
      $return .= action_list($config);
    }
  }

  return($return);
}

?>
